# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("stories", "0011_story_audio_link")]

    operations = [
        migrations.AddField(
            model_name="story",
            name="written_by",
            field=models.CharField(help_text="The writer of the story", max_length=255, null=True, blank=True),
        )
    ]



# -*- coding: utf-8 -*-


from django.db import migrations


def add_category_to_current_stories(apps, schema_editor):
    Story = apps.get_model("stories", "Story")
    Category = apps.get_model("categories", "Category")

    for story in Story.objects.all():
        general_category = Category.objects.get(name__icontains="general", org=story.org)
        story.category = general_category
        story.save()


class Migration(migrations.Migration):

    dependencies = [("stories", "0005_story_category")]

    operations = [migrations.RunPython(add_category_to_current_stories)]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("stories", "0001_initial")]

    operations = [
        migrations.AlterField(
            model_name="story",
            name="image",
            field=models.ImageField(
                help_text="Any image that should be displayed with this story",
                null=True,
                upload_to="stories",
                blank=True,
            ),
        )
    ]



import inspect
import json
import logging
from functools import wraps

from celery import shared_task, signature
from django_redis import get_redis_connection

from django.apps import apps
from django.utils import timezone

from .models import Invitation, TaskState

ORG_TASK_LOCK_KEY = "org-task-lock:%s:%s"

logger = logging.getLogger(__name__)


@shared_task(track_started=True, name="send_invitation_email_task")
def send_invitation_email_task(invitation_id):
    invitation = Invitation.objects.get(pk=invitation_id)
    invitation.send_email()


@shared_task
def trigger_org_task(task_name, queue="celery"):
    """
    Triggers the given org task to be run for all active orgs
    :param task_name: the full task name, e.g. 'myproj.myapp.tasks.do_stuff'
    :param queue: the name of the queue to send org sub-tasks to
    """
    active_orgs = apps.get_model("orgs", "Org").objects.filter(is_active=True)
    for org in active_orgs:
        sig = signature(task_name, args=[org.pk])
        sig.apply_async(queue=queue)

    logger.info("Requested task '%s' for %d active orgs" % (task_name, len(active_orgs)))


def org_task(task_key, lock_timeout=None):
    """
    Decorator to create an org task.
    :param task_key: the task key used for state storage and locking, e.g. 'do-stuff'
    :param lock_timeout: the lock timeout in seconds
    """

    def _org_task(task_func):
        def _decorator(org_id):
            org = apps.get_model("orgs", "Org").objects.get(pk=org_id)
            maybe_run_for_org(org, task_func, task_key, lock_timeout)

        return shared_task(wraps(task_func)(_decorator))

    return _org_task


def maybe_run_for_org(org, task_func, task_key, lock_timeout):
    """
    Runs the given task function for the specified org provided it's not already running
    :param org: the org
    :param task_func: the task function
    :param task_key: the task key
    :param lock_timeout: the lock timeout in seconds
    """
    r = get_redis_connection()

    key = TaskState.get_lock_key(org, task_key)

    if r.get(key):
        logger.warning("Skipping task %s for org #%d as it is still running" % (task_key, org.id))
    else:
        with r.lock(key, timeout=lock_timeout):
            state = org.get_task_state(task_key)
            if state.is_disabled:
                logger.info("Skipping task %s for org #%d as is marked disabled" % (task_key, org.id))
                return

            logger.info("Started task %s for org #%d..." % (task_key, org.id))

            prev_results = json.loads(state.last_results) if state.last_results else None
            prev_started_on = state.last_successfully_started_on
            this_started_on = timezone.now()

            state.started_on = this_started_on
            state.ended_on = None
            state.save(update_fields=("started_on", "ended_on"))

            num_task_args = len(inspect.getfullargspec(task_func).args)

            assert num_task_args >= 1, "task signature must be foo(org) or foo(org, since, until)"

            task_args = [org]

            try:
                if num_task_args >= 3:
                    task_args += [prev_started_on, this_started_on]
                if num_task_args >= 4:
                    task_args.append(prev_results)

                results = task_func(*task_args)

                state.ended_on = timezone.now()
                state.last_successfully_started_on = this_started_on
                state.last_results = json.dumps(results)
                state.is_failing = False
                state.save(update_fields=("ended_on", "last_successfully_started_on", "last_results", "is_failing"))

                logger.info("Finished task %s for org #%d with result: %s" % (task_key, org.id, json.dumps(results)))

            except Exception as e:
                state.ended_on = timezone.now()
                state.last_results = None
                state.is_failing = True
                state.save(update_fields=("ended_on", "last_results", "is_failing"))

                logger.exception("Task %s for org #%d failed" % (task_key, org.id))
                raise e  # re-raise with original stack trace



# Generated by Django 2.2.19 on 2021-08-26 10:37

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("orgs", "0026_fix_org_config_rapidpro"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Tag",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                        editable=False,
                        help_text="When this item was originally created",
                    ),
                ),
                (
                    "modified_on",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                        editable=False,
                        help_text="When this item was last modified",
                    ),
                ),
                ("name", models.CharField(help_text="The name of this tag", max_length=64)),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="tags_tag_creations",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="tags_tag_modifications",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "org",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="orgs.Org",
                    ),
                ),
            ],
            options={
                "unique_together": {("name", "org")},
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0011_auto_20150710_1612")]

    operations = [
        migrations.AlterField(
            model_name="invitation",
            name="email",
            field=models.EmailField(
                help_text="The email to which we send the invitation of the viewer",
                max_length=254,
                verbose_name="Email",
            ),
        )
    ]



from smartmin.views import SmartCreateView, SmartCRUDL, SmartListView, SmartUpdateView

from django import forms

from dash.categories.fields import CategoryChoiceField
from dash.orgs.views import OrgObjPermsMixin, OrgPermsMixin

from .models import Category, CategoryImage


class CategoryImageForm(forms.ModelForm):
    category = CategoryChoiceField(Category.objects.none())

    def __init__(self, *args, **kwargs):
        self.org = kwargs["org"]
        del kwargs["org"]

        super(CategoryImageForm, self).__init__(*args, **kwargs)
        self.fields["category"].queryset = Category.objects.filter(org=self.org).order_by("name")

    class Meta:
        model = CategoryImage
        fields = ("is_active", "name", "category", "image")


class CategoryCRUDL(SmartCRUDL):
    model = Category
    actions = ("create", "update", "list")

    class Update(OrgObjPermsMixin, SmartUpdateView):
        fields = ("is_active", "name")

    class List(OrgPermsMixin, SmartListView):
        ordering = ("name",)

        def derive_fields(self):
            if self.request.user.is_superuser:
                return ("name", "modified_on", "created_on", "org")
            return ("name", "modified_on", "created_on")

        def get_queryset(self, **kwargs):
            queryset = super(CategoryCRUDL.List, self).get_queryset(**kwargs)
            queryset = queryset.filter(org=self.derive_org())

            return queryset

    class Create(OrgPermsMixin, SmartCreateView):
        def derive_fields(self):
            if self.request.user.is_superuser:
                return ("name", "org")
            return ("name",)

        def pre_save(self, obj):
            obj = super(CategoryCRUDL.Create, self).pre_save(obj)

            if not self.get_user().is_superuser:
                org = self.derive_org()
                if org:
                    obj.org = org

            return obj


class CategoryImageCRUDL(SmartCRUDL):
    model = CategoryImage
    actions = ("create", "update", "list")

    class Update(OrgObjPermsMixin, SmartUpdateView):
        form_class = CategoryImageForm
        fields = ("is_active", "name", "category", "image")

        def get_object_org(self):
            return self.get_object().category.org

        def get_form_kwargs(self):
            kwargs = super(CategoryImageCRUDL.Update, self).get_form_kwargs()
            kwargs["org"] = self.request.org
            return kwargs

    class List(OrgPermsMixin, SmartListView):
        fields = ("name", "category", "modified_on", "created_on")
        ordering = ("name", "category")

        def get_queryset(self, **kwargs):
            queryset = super(CategoryImageCRUDL.List, self).get_queryset(**kwargs)
            queryset = queryset.filter(category__org=self.derive_org())

            return queryset

    class Create(OrgPermsMixin, SmartCreateView):
        form_class = CategoryImageForm
        fields = ("name", "category", "image")

        def get_form_kwargs(self):
            kwargs = super(CategoryImageCRUDL.Create, self).get_form_kwargs()
            kwargs["org"] = self.request.org
            return kwargs



import itertools
import json

import redis
import requests
from requests.structures import CaseInsensitiveDict

from django.contrib.auth.models import User
from django.http import JsonResponse
from django.test import TestCase

from dash.orgs.models import Org
from dash.utils import random_string


class MockResponse:
    """
    Mock response object with a status code and some content
    """

    def __init__(self, status_code, content=None, headers=None):
        self.status_code = status_code
        self.content = content or ""
        self.headers = CaseInsensitiveDict()

        if headers:
            self.headers.update(headers)

    def raise_for_status(self):
        http_error_msg = ""

        if 400 <= self.status_code < 500:
            http_error_msg = "%s Client Error: ..." % self.status_code

        elif 500 <= self.status_code < 600:
            http_error_msg = "%s Server Error: ..." % self.status_code

        if http_error_msg:
            raise requests.HTTPError(http_error_msg, response=self)

    def json(self, **kwargs):
        return json.loads(self.content)


class DashTest(TestCase):
    """
    Base class for dashboard test cases
    """

    def setUp(self):
        super(DashTest, self).setUp()

        self.superuser = User.objects.create_superuser(username="testroot", email="super@user.com", password="root")

        self.clear_cache()

    @classmethod
    def clear_cache(cls):
        # we are extra paranoid here and actually hardcode redis to 'localhost' and '10'
        # Redis 10 is our testing redis db
        r = redis.StrictRedis(host="localhost", db=10)
        r.flushdb()

    def create_org(self, name, timezone, subdomain):
        org = Org.objects.create(
            name=name, timezone=timezone, subdomain=subdomain, created_by=self.superuser, modified_by=self.superuser
        )
        org.backends.get_or_create(
            api_token=random_string(32), slug="rapidpro", created_by=self.superuser, modified_by=self.superuser
        )
        return org

    def login(self, user):
        password = "root" if user == self.superuser else user.username
        result = self.client.login(username=user.username, password=password)
        self.assertTrue(result, "Couldn't login as %(user)s / %(user)s" % dict(user=user.username))

    def url_get(self, subdomain, url, params=None, **extra):
        return self._url_request(subdomain, "get", url, params, **extra)

    def url_post(self, subdomain, url, data=None, **extra):
        return self._url_request(subdomain, "post", url, data, **extra)

    def assertLoginRedirect(self, response, subdomain, next_url):
        self.assertRedirects(response, "http://%s.localhost/users/login/?next=%s" % (subdomain, next_url))

    def _url_request(self, subdomain, method, url, data, **extra):
        if data is None:
            data = {}

        if subdomain:
            extra["HTTP_HOST"] = "%s.localhost" % subdomain

        func = getattr(self.client, method)
        response = func(url, data, **extra)

        if isinstance(response, JsonResponse):
            content = response.content
            if isinstance(content, bytes):
                content = content.decode("utf-8")

            response.json = json.loads(content)

        return response


class MockClientQuery:
    """
    Mock for APIv2 client get_xxxxx return values. Pass lists of temba objects to mock each fetch the client would make.

    For example:
        mock_get_contacts.return_value = MockClientQuery(
            [TembaContact.create(...), TembaContact.create(...), TembaContact.create(...)]
            [TembaContact.create(...)]
        )

    Will return the three contacts on the first call to iterfetches, and one on the second call.

    """

    def __init__(self, *fetches):
        self.fetches = list(fetches)

    def iterfetches(self, *args, **kwargs):
        return self

    def all(self, *args, **kwargs):
        return list(itertools.chain.from_iterable(self.fetches))

    def first(self, *args, **kwargs):
        return self.fetches[0][0] if self.fetches[0] else None

    def get_cursor(self):
        return "cursor-string"

    def __iter__(self):
        return self

    def __next__(self):
        if not self.fetches:
            raise StopIteration()

        return self.fetches.pop(0)



# -*- coding: utf-8 -*-


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("stories", "0008_auto_20140912_0524")]

    operations = [migrations.RemoveField(model_name="story", name="image")]



# -*- coding: utf-8 -*-


from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0006_auto_20140919_2056")]

    operations = [
        migrations.AlterField(
            model_name="invitation",
            name="created_by",
            field=models.ForeignKey(
                related_name="orgs_invitation_creations",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="invitation",
            name="modified_by",
            field=models.ForeignKey(
                related_name="orgs_invitation_modifications",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which last modified this item",
            ),
        ),
        migrations.AlterField(
            model_name="invitation",
            name="org",
            field=models.ForeignKey(
                related_name="invitations",
                verbose_name="Org",
                on_delete=models.PROTECT,
                to="orgs.Org",
                help_text="The organization to which the account is invited to view",
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="administrators",
            field=models.ManyToManyField(
                help_text="The administrators in your organization",
                related_name="org_admins",
                verbose_name="Administrators",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="created_by",
            field=models.ForeignKey(
                related_name="orgs_org_creations",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="editors",
            field=models.ManyToManyField(
                help_text="The editors in your organization",
                related_name="org_editors",
                verbose_name="Editors",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="modified_by",
            field=models.ForeignKey(
                related_name="orgs_org_modifications",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which last modified this item",
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="viewers",
            field=models.ManyToManyField(
                help_text="The viewers in your organization",
                related_name="org_viewers",
                verbose_name="Viewers",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="orgbackground",
            name="created_by",
            field=models.ForeignKey(
                related_name="orgs_orgbackground_creations",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="orgbackground",
            name="modified_by",
            field=models.ForeignKey(
                related_name="orgs_orgbackground_modifications",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which last modified this item",
            ),
        ),
        migrations.AlterField(
            model_name="orgbackground",
            name="org",
            field=models.ForeignKey(
                related_name="backgrounds",
                verbose_name="Org",
                on_delete=models.PROTECT,
                to="orgs.Org",
                help_text="The organization in which the image will be used",
            ),
        ),
    ]



from django.db import migrations


def generate_initial_block_types(apps, schema_editor):
    User = apps.get_model("auth", "User")
    root = User.objects.filter(username="root").first()

    if not root:
        root = User.objects.filter(username="root2").first()

    if not root:
        root = User.objects.create(username="root2")

    DashBlockType = apps.get_model("dashblocks", "DashBlockType")

    DashBlockType.objects.get_or_create(
        name="Missions",
        slug="missions",
        description="U-Report missions blocks",
        has_title=True,
        has_image=True,
        has_rich_text=False,
        has_summary=False,
        has_link=False,
        has_gallery=False,
        has_color=False,
        has_video=False,
        has_tags=False,
        created_by=root,
        modified_by=root,
    )

    DashBlockType.objects.get_or_create(
        name="Contact Us",
        slug="contact_us",
        description="U-Report contact us blocks",
        has_title=True,
        has_image=False,
        has_rich_text=False,
        has_summary=False,
        has_link=False,
        has_gallery=False,
        has_color=False,
        has_video=False,
        has_tags=False,
        created_by=root,
        modified_by=root,
    )


class Migration(migrations.Migration):

    dependencies = [("dashblocks", "0003_auto_20140804_0236")]

    operations = [migrations.RunPython(generate_initial_block_types)]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.10 on 2016-10-26 15:13


import timezone_field.fields

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("orgs", "0016_taskstate_is_disabled")]

    operations = [
        migrations.AlterField(
            model_name="org",
            name="timezone",
            field=timezone_field.fields.TimeZoneField(
                default="UTC", help_text="The timezone your organization is in.", verbose_name="Timezone"
            ),
        )
    ]



from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0005_orgbackground"), migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="Category",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(help_text="When this item was originally created", auto_now_add=True),
                ),
                ("modified_on", models.DateTimeField(help_text="When this item was last modified", auto_now=True)),
                ("name", models.CharField(help_text="The name of this category", max_length=64)),
                (
                    "image",
                    models.ImageField(
                        help_text="An optional image that can describe this category",
                        null=True,
                        upload_to="categories",
                        blank=True,
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "org",
                    models.ForeignKey(
                        help_text="The organization this category applies to", on_delete=models.PROTECT, to="orgs.Org"
                    ),
                ),
            ],
            options={"verbose_name_plural": "Categories"},
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(name="category", unique_together=set([("name", "org")])),
    ]



import time
from abc import ABCMeta, abstractmethod
from enum import Enum
from typing import Optional, Tuple

"""
Sync support
"""


class SyncOutcome(Enum):
    created = 1
    updated = 2
    deleted = 3
    ignored = 4


class BaseSyncer(object, metaclass=ABCMeta):
    """
    Base class for classes that describe how to synchronize particular local models against incoming data
    """

    model = None
    local_id_attr = "uuid"
    remote_id_attr = "uuid"
    select_related = ()
    prefetch_related = ()
    local_backend_attr = None

    def __init__(self, backend):
        self.backend = backend

    def identify_local(self, local):
        """
        Gets the unique identity of the local model instance
        :param local: the local model instance
        :return: the unique identity
        """
        return getattr(local, self.local_id_attr)

    def identify_remote(self, remote):
        """
        Gets the unique identity of the remote object
        :param remote: the remote object
        :return: the unique identity
        """
        return getattr(remote, self.remote_id_attr)

    def lock(self, org, identity):
        """
        Gets a lock on the given identity value
        :param org: the org
        :param identity: the unique identity
        :return: the lock
        """
        return self.model.lock(org, identity)

    def fetch_local(self, org, identity):
        """
        Fetches the local model instance with the given identity, returning none if it doesn't exist
        :param org: the org
        :param identity: the unique identity
        :return: the instance or none
        """
        qs = self.fetch_all(org=org).filter(**{self.local_id_attr: identity})

        if self.select_related:
            qs = qs.select_related(*self.select_related)
        if self.prefetch_related:
            qs = qs.prefetch_related(*self.prefetch_related)

        return qs.first()

    def fetch_all(self, org):
        """
        Fetches all local objects
        :param org: the org
        :return: the queryset
        """
        qs = self.model.objects.filter(org=org)
        if self.local_backend_attr is not None:
            qs = qs.filter(**{self.local_backend_attr: self.backend})
        return qs

    @abstractmethod
    def local_kwargs(self, org, remote):
        """
        Generates kwargs for creating or updating a local model instance from a remote object. Returning none from this
        method means that the remote object doesn't belong locally.
        :param org: the org
        :param remote: the incoming remote object
        :return: the kwargs or none
        """
        pass

    def update_required(self, local, remote, remote_as_kwargs):
        """
        Determines whether local instance differs from the remote object and so needs to be updated. By default this
        will always update the local instance which is obviously inefficient.
        :param local: the local instance
        :param remote: the incoming remote object
        :param remote_as_kwargs: the generated kwargs from remote object
        :return: whether the local instance must be updated
        """
        return True

    def delete_local(self, local):
        """
        Deletes a local instance
        :param local:
        :return:
        """
        local.is_active = False
        local.save(update_fields=("is_active",))

    def create_local(self, remote_as_kwargs):
        """
        Create a local instance
        :param remote_as_kwargs: the generated kwargs from remote object
        :return: the created instance
        """
        return self.model.objects.create(**remote_as_kwargs)

    def update_local(self, local, remote_as_kwargs):
        """
        Update a local instance
        :param local: the local instance
        :param remote_as_kwargs: the generated kwargs from remote object
        :return: the updated instance
        """
        for field, value in remote_as_kwargs.items():
            setattr(local, field, value)
        local.is_active = True
        local.save()
        return local


def sync_from_remote(org, syncer, remote):
    """
    Sync local instance against a single remote object

    :param * org: the org
    :param * syncer: the local model syncer
    :param * remote: the remote object
    :return: the outcome (created, updated or deleted)
    """
    identity = syncer.identify_remote(remote)

    with syncer.lock(org, identity):
        existing = syncer.fetch_local(org, identity)

        # derive kwargs for the local model (none return here means don't keep)
        remote_as_kwargs = syncer.local_kwargs(org, remote)

        # exists locally
        if existing:
            existing.org = org  # saves pre-fetching since we already have the org

            if remote_as_kwargs:
                if syncer.update_required(existing, remote, remote_as_kwargs) or not existing.is_active:
                    syncer.update_local(existing, remote_as_kwargs)
                    return SyncOutcome.updated

            elif existing.is_active:  # exists locally, but shouldn't now to due to model changes
                syncer.delete_local(existing)
                return SyncOutcome.deleted

        elif remote_as_kwargs:
            syncer.create_local(remote_as_kwargs)
            return SyncOutcome.created

    return SyncOutcome.ignored


def sync_local_to_set(org, syncer, remote_set) -> dict:
    """
    Syncs an org's set of local instances of a model to match the set of remote objects. Local objects not in the remote
    set are deleted.

    :param org: the org
    :param * syncer: the local model syncer
    :param remote_set: the set of remote objects
    :return: dict of counts of created, updated, deleted, ignored local instances
    """
    outcome_counts = {SyncOutcome.created: 0, SyncOutcome.updated: 0, SyncOutcome.deleted: 0, SyncOutcome.ignored: 0}

    remote_identities = set()

    for remote in remote_set:
        outcome = sync_from_remote(org, syncer, remote)
        outcome_counts[outcome] += 1

        remote_identities.add(syncer.identify_remote(remote))

    # active local objects which weren't in the remote set need to be deleted
    active_locals = syncer.fetch_all(org).filter(is_active=True)
    delete_locals = active_locals.exclude(**{syncer.local_id_attr + "__in": remote_identities})

    for local in delete_locals:
        with syncer.lock(org, syncer.identify_local(local)):
            syncer.delete_local(local)
            outcome_counts[SyncOutcome.deleted] += 1

    return outcome_counts


def sync_local_to_changes(
    org, syncer, fetches, deleted_fetches, progress_callback=None, time_limit: int = None
) -> Tuple[dict, Optional[str]]:
    """
    Sync local instances against iterators which return fetches of changed and deleted remote objects.

    :param * org: the org
    :param * syncer: the local model syncer
    :param * fetches: an iterator returning fetches of modified remote objects
    :param * deleted_fetches: an iterator returning fetches of deleted remote objects
    :param * progress_callback: callable for tracking progress - called for each fetch with number of contacts fetched
    :param * time_limit: number of seconds to limit fetching too (optional)
    :return: tuple of a dict of counts of created, updated, deleted, ignored local instances and a possible cursor if
             fetching didn't complete
    """
    num_synced = 0
    outcome_counts = {SyncOutcome.created: 0, SyncOutcome.updated: 0, SyncOutcome.deleted: 0, SyncOutcome.ignored: 0}
    resume_cursor = None

    start = time.time()

    for fetch in fetches:
        for remote in fetch:
            outcome = sync_from_remote(org, syncer, remote)
            outcome_counts[outcome] += 1

        num_synced += len(fetch)
        if progress_callback:
            progress_callback(num_synced)

        if time_limit and time.time() - start > time_limit:
            resume_cursor = fetches.get_cursor()
            break

    # any item that has been deleted remotely should also be released locally
    for deleted_fetch in deleted_fetches:
        for deleted_remote in deleted_fetch:
            identity = syncer.identify_remote(deleted_remote)
            with syncer.lock(org, identity):
                existing = syncer.fetch_local(org, identity)
                if existing:
                    syncer.delete_local(existing)
                    outcome_counts[SyncOutcome.deleted] += 1

        num_synced += len(deleted_fetch)
        if progress_callback:
            progress_callback(num_synced)

    return outcome_counts, resume_cursor



from django import template

register = template.Library()


@register.simple_tag(takes_context=True)
def if_url(context, url_name, yes, no):
    """
    Example:
        %li{ class:"{% if_url 'contacts.contact_read' 'active' '' %}" }
    """
    current = context["request"].resolver_match.url_name
    return yes if url_name == current else no



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0010_auto_20150618_1042")]

    operations = [
        migrations.AddField(
            model_name="org",
            name="domain",
            field=models.CharField(
                null=True,
                error_messages={"unique": "This domain is not available"},
                max_length=255,
                blank=True,
                help_text="The custom domain for this U-Report instance",
                unique=True,
                verbose_name="Domain",
            ),
            preserve_default=True,
        )
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0005_orgbackground")]

    operations = [
        migrations.AlterField(
            model_name="org",
            name="language",
            field=models.CharField(
                choices=[("en", "English"), ("fr", "French")],
                max_length=64,
                blank=True,
                help_text="The main language used by this organization",
                null=True,
                verbose_name="Language",
            ),
        )
    ]



# -*- coding: utf-8 -*-


from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="Invitation",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(help_text="When this item was originally created", auto_now_add=True),
                ),
                ("modified_on", models.DateTimeField(help_text="When this item was last modified", auto_now=True)),
                (
                    "email",
                    models.EmailField(
                        help_text="The email to which we send the invitation of the viewer",
                        max_length=75,
                        verbose_name="Email",
                    ),
                ),
                (
                    "secret",
                    models.CharField(
                        help_text="a unique code associated with this invitation",
                        unique=True,
                        max_length=64,
                        verbose_name="Secret",
                    ),
                ),
                (
                    "user_group",
                    models.CharField(
                        default="V",
                        max_length=1,
                        verbose_name="User Role",
                        choices=[("A", "Administrator"), ("E", "Editor"), ("V", "Viewer")],
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"abstract": False},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="Org",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(help_text="When this item was originally created", auto_now_add=True),
                ),
                ("modified_on", models.DateTimeField(help_text="When this item was last modified", auto_now=True)),
                (
                    "name",
                    models.CharField(help_text="The name of this organization", max_length=128, verbose_name="Name"),
                ),
                (
                    "language",
                    models.CharField(
                        choices=[("en-us", "English"), ("rw", "Kinyarwanda"), ("fr", "French")],
                        max_length=64,
                        blank=True,
                        help_text="The main language used by this organization",
                        null=True,
                        verbose_name="Language",
                    ),
                ),
                (
                    "subdomain",
                    models.SlugField(
                        error_messages={"unique": "This subdomain is not available"},
                        max_length=255,
                        help_text="The subdomain for this UReport instance",
                        unique=True,
                        verbose_name="Subdomain",
                    ),
                ),
                (
                    "api_token",
                    models.CharField(
                        help_text="The API token for the RapidPro account this dashboard is tied to",
                        max_length=128,
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "config",
                    models.TextField(
                        help_text="JSON blob used to store configuration information associated with this organization",
                        null=True,
                        blank=True,
                    ),
                ),
                ("administrators", models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name="Administrators")),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("editors", models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name="Editors")),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("viewers", models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name="Viewers")),
            ],
            options={"abstract": False},
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name="invitation",
            name="org",
            field=models.ForeignKey(
                verbose_name="Org",
                to="orgs.Org",
                on_delete=models.PROTECT,
                help_text="The organization to which the account is invited to view",
            ),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations


def populate_story_summary(apps, schema_editor):
    Story = apps.get_model("stories", "Story")
    for story in Story.objects.all():
        story.summary = story.content
        story.save()


class Migration(migrations.Migration):

    dependencies = [("stories", "0003_story_summary")]

    operations = [migrations.RunPython(populate_story_summary)]



from django.conf import settings


def lang_direction(request):
    """
    Sets lang_direction context variable to whether the language is RTL or LTR
    """
    if lang_direction.rtl_langs is None:
        lang_direction.rtl_langs = getattr(settings, "RTL_LANGUAGES", set())

    return {"lang_direction": "rtl" if request.LANGUAGE_CODE in lang_direction.rtl_langs else "ltr"}


lang_direction.rtl_langs = None



from django import template
from django.conf import settings
from django.db.models import Prefetch

from dash.dashblocks.models import DashBlock, DashBlockType

"""
This module offers one templatetag called ``load_dashblocks``.

``load_dashblocks`` does a query for all active DashBlock objects
for the passed in DashBlockType and Org on request. (identified by the slug)
You can then access that list within your context.

It accepts 2 parameter:

    org
        The Org set on the request to filter DashBlocks for that org.

    slug
        The slug/key of the DashBlockType to load DashBlocks for.

        If you want to pass it by name, you have to use quotes on it.
        Otherwise just use the variable name.

Syntax::

    {% load_dashblocks [org] [name] %}

Example usage::

    {% load dashblocks %}

    ...

    {% load_dashblocks request.org "home_banner_blocks" %}

    ...

    Note: You may also use the shortcut tag 'load_qbs'
    eg: {% load_qbs request.org "home_banner_blocks %}

.. note::

    If you specify a slug that has no associated dash block, then an error message
    will be inserted in your template.  You may change this text by setting
    the value of the DASHBLOCK_STRING_IF_INVALID setting.

"""


register = template.Library()


@register.simple_tag(takes_context=True)
def load_dashblocks(context, org, slug, tag=None):
    if not org:
        return ""

    dashblocks_qs = DashBlock.objects.filter(org=org, is_active=True).order_by("-priority")
    # filter by our tag if one was specified
    if tag is not None:
        dashblocks_qs = dashblocks_qs.filter(tags__icontains=tag)

    dashblock_type = (
        DashBlockType.objects.filter(slug=slug)
        .prefetch_related(
            Prefetch(
                "dashblock_set",
                queryset=dashblocks_qs,
                to_attr="prefetch_dashblocks",
            )
        )
        .first()
    )

    if not dashblock_type:
        default_invalid = '<b><font color="red">DashBlockType with slug: %s not found</font></b>'
        return getattr(settings, "DASHBLOCK_STRING_IF_INVALID", default_invalid) % slug

    dashblocks = dashblock_type.prefetch_dashblocks

    context[slug] = dashblocks

    return ""


@register.simple_tag(takes_context=True)
def load_qbs(context, org, slug, tag=None):
    return load_dashblocks(context, org, slug, tag)



# -*- coding: utf-8 -*-


from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("stories", "0009_remove_story_image")]

    operations = [
        migrations.AlterField(
            model_name="story",
            name="created_by",
            field=models.ForeignKey(
                related_name="stories_story_creations",
                on_delete=models.PROTECT,
                to=settings.AUTH_USER_MODEL,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="story",
            name="modified_by",
            field=models.ForeignKey(
                related_name="stories_story_modifications",
                on_delete=models.PROTECT,
                to=settings.AUTH_USER_MODEL,
                help_text="The user which last modified this item",
            ),
        ),
        migrations.AlterField(
            model_name="story",
            name="video_id",
            field=models.CharField(
                help_text="The id of the YouTube video that should be linked to this story (this is the text that comes afer v= and before & in the YouTube URL)",
                max_length=255,
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="storyimage",
            name="created_by",
            field=models.ForeignKey(
                on_delete=models.PROTECT,
                related_name="stories_storyimage_creations",
                to=settings.AUTH_USER_MODEL,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="storyimage",
            name="modified_by",
            field=models.ForeignKey(
                on_delete=models.PROTECT,
                related_name="stories_storyimage_modifications",
                to=settings.AUTH_USER_MODEL,
                help_text="The user which last modified this item",
            ),
        ),
        migrations.AlterField(
            model_name="storyimage",
            name="story",
            field=models.ForeignKey(
                related_name="images",
                on_delete=models.PROTECT,
                to="stories.Story",
                help_text="The story to associate to",
            ),
        ),
    ]



# Generated by Django 3.2.6 on 2021-09-02 11:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("orgs", "0026_fix_org_config_rapidpro"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="org",
            options={"ordering": ["name"]},
        ),
    ]



from smartmin.views import SmartCreateView, SmartCRUDL, SmartListView, SmartUpdateView

from django import forms
from django.urls import reverse
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from dash.orgs.views import OrgObjPermsMixin, OrgPermsMixin

from .models import DashBlock, DashBlockImage, DashBlockType


class DashBlockTypeCRUDL(SmartCRUDL):
    model = DashBlockType
    actions = ("create", "update", "list")

    class List(SmartListView):
        title = _("Content Types")
        fields = ("name", "slug", "description")
        link_fields = ("name",)
        ordering = ("name",)


class DashBlockFormMixin(object):
    def get_type(self):
        block_type = self.request.POST.get("type", self.request.GET.get("type", None))
        if block_type:
            return DashBlockType.objects.filter(id=block_type).first()
        return None

    def get_context_data(self, *args, **kwargs):
        context = super(DashBlockFormMixin, self).get_context_data(*args, **kwargs)
        context["type"] = self.get_type()
        return context

    def get_success_url(self):
        url = reverse("dashblocks.dashblock_list")
        return "%s?type=%d" % (url, self.object.dashblock_type.id)

    def derive_exclude(self):
        exclude = super(DashBlockFormMixin, self).derive_exclude()

        block_type = self.get_type()
        if block_type:
            if not self.request.user.has_perm(self.permission):
                exclude.append("dashblock_type")

            if not block_type.has_summary:
                exclude.append("summary")

            if not block_type.has_video:
                exclude.append("video_id")

            if not block_type.has_title:
                exclude.append("title")

            if not block_type.has_tags:
                exclude.append("tags")

            if not block_type.has_image:
                exclude.append("image")

            if not block_type.has_link:
                exclude.append("link")

            if not block_type.has_color:
                exclude.append("color")

        return exclude

    def pre_save(self, obj):
        obj = super(DashBlockFormMixin, self).pre_save(obj)

        block_type = self.get_type()
        if block_type:
            obj.dashblock_type = block_type

        obj.org = self.request.org

        obj.space_tags()
        return obj


class DashBlockForm(forms.ModelForm):
    class Meta:
        model = DashBlock
        fields = (
            "title",
            "summary",
            "content",
            "image",
            "color",
            "link",
            "video_id",
            "tags",
            "dashblock_type",
            "priority",
            "is_active",
        )
        labels = {"link": _("Link Slug")}
        help_texts = {"link": _("The slug or word to use in the link of the page")}


class DashBlockCRUDL(SmartCRUDL):
    model = DashBlock
    permissions = True
    actions = ("create", "update", "list")

    class List(OrgPermsMixin, SmartListView):
        fields = ("title", "priority", "dashblock_type", "tags")
        link_fields = ("title",)
        default_order = "-modified_on"
        ordering = ("-modified_on",)
        search_fields = ("title__icontains", "content__icontains", "summary__icontains")

        def derive_fields(self):
            fields = super(DashBlockCRUDL.List, self).derive_fields()
            block_type = self.get_type()
            if block_type:
                if not block_type.has_tags:
                    fields = type(fields)(x for x in fields if x != "tags")
            return fields

        def get_title(self, obj):
            block_type = self.get_type()
            if block_type:
                if not block_type.has_title:
                    return force_str(obj)
            return obj.title

        def derive_title(self):
            type = self.get_type()
            if not type:
                return _("Content Blocks")
            else:
                return _("%s Blocks") % type.name

        def get_type(self):
            block_type = self.request.POST.get("type", self.request.GET.get("type", None))
            if block_type and block_type != "0":
                return DashBlockType.objects.filter(id=block_type).first()
            slug = self.request.POST.get("slug", self.request.GET.get("slug", None))
            if slug:
                return DashBlockType.objects.filter(slug=slug).first()
            return None

        def get_queryset(self, **kwargs):
            queryset = super(DashBlockCRUDL.List, self).get_queryset(**kwargs)

            dashblock_type = self.get_type()
            if dashblock_type:
                queryset = queryset.filter(dashblock_type=dashblock_type)

            queryset = queryset.filter(dashblock_type__is_active=True)
            queryset = queryset.filter(org=self.request.org)

            return queryset

        def get_context_data(self, *args, **kwargs):
            context = super(DashBlockCRUDL.List, self).get_context_data(*args, **kwargs)
            context["types"] = DashBlockType.objects.filter(is_active=True)
            context["filtered_type"] = self.get_type()
            return context

    class Update(OrgObjPermsMixin, DashBlockFormMixin, SmartUpdateView):
        form_class = DashBlockForm
        fields = (
            "title",
            "summary",
            "content",
            "image",
            "color",
            "link",
            "video_id",
            "tags",
            "dashblock_type",
            "priority",
            "is_active",
        )

        def get_type(self):
            return self.object.dashblock_type

        def derive_title(self):
            return _("Edit %s") % self.get_type().name

    class Create(OrgPermsMixin, DashBlockFormMixin, SmartCreateView):
        form_class = DashBlockForm
        grant_permissions = ("dashblocks.change_dashblock",)

        def derive_initial(self, *args, **kwargs):
            initial = super(DashBlockCRUDL.Create, self).derive_initial(*args, **kwargs)
            dashblock_type = self.get_type()
            other_blocks = DashBlock.objects.filter(
                is_active=True, org=self.derive_org(), dashblock_type=dashblock_type
            )
            other_blocks = other_blocks.order_by("-priority")
            if not other_blocks:
                initial["priority"] = 0
            else:
                initial["priority"] = other_blocks[0].priority + 1

            return initial

        def derive_title(self):
            block_type = self.get_type()
            if block_type:
                return _("Create %s") % block_type.name
            else:
                return _("Create Content Block")


class DashBlockImageCRUDL(SmartCRUDL):
    model = DashBlockImage
    actions = ("create", "update", "list")

    class List(SmartListView):
        ordering = ("dashblock__title",)

    class Update(SmartUpdateView):
        exclude = ("dashblock", "modified_by", "modified_on", "created_on", "created_by", "width", "height")
        title = "Edit Image"
        success_message = "Image edited successfully."

        def get_success_url(self):
            return reverse("dashblocks.dashblock_update", args=[self.object.dashblock.id])

    class Create(SmartCreateView):
        exclude = (
            "dashblock",
            "is_active",
            "modified_by",
            "modified_on",
            "created_on",
            "created_by",
            "width",
            "height",
        )
        title = "Add Image"
        success_message = "Image added successfully."

        def derive_initial(self, *args, **kwargs):
            initial = super(DashBlockImageCRUDL.Create, self).derive_initial(*args, **kwargs)
            block_id = self.request.POST.get("dashblock", self.request.GET.get("dashblock", None))
            dashblock = DashBlock.objects.get(pk=block_id)
            images = dashblock.sorted_images()
            if not images:
                initial["priority"] = 0
            else:
                initial["priority"] = images[0].priority + 1
            return initial

        def get_success_url(self):
            return reverse("dashblocks.dashblock_update", args=[self.object.dashblock.id])

        def pre_save(self, obj):
            obj = super(DashBlockImageCRUDL.Create, self).pre_save(obj)
            block_id = self.request.POST.get("dashblock", self.request.GET.get("dashblock", None))
            obj.dashblock = DashBlock.objects.filter(pk=block_id).first()
            return obj



from . import views

urlpatterns = views.OrgCRUDL().as_urlpatterns()
urlpatterns += views.OrgBackgroundCRUDL().as_urlpatterns()
urlpatterns += views.TaskCRUDL().as_urlpatterns()
urlpatterns += views.OrgBackendCRUDL().as_urlpatterns()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-07 08:24


import json

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("orgs", "0018_auto_20170301_0914")]

    def migrate_api_token_and_common_org_config(apps, schema_editor):
        Org = apps.get_model("orgs", "Org")
        orgs = Org.objects.all()

        for org in orgs:
            if not org.config:
                old_config = dict()
            else:
                old_config = json.loads(org.config)

            if "common" in old_config and "rapidpro" in old_config:
                print("Skipped org(%d), it looks like already migrated" % org.id)
                continue

            new_config = {"common": old_config, "rapidpro": {"api_token": org.api_token}}
            org.config = json.dumps(new_config)
            org.save()

    def noop(apps, schema_editor):
        pass

    operations = [migrations.RunPython(migrate_api_token_and_common_org_config, noop)]



from timezone_field import TimeZoneFormField

from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

from .models import Org


class CreateOrgLoginForm(forms.Form):
    first_name = forms.CharField(help_text=_("Your first name"))
    last_name = forms.CharField(help_text=_("Your last name"))
    email = forms.EmailField(help_text=_("Your email address"))
    password = forms.CharField(widget=forms.PasswordInput, help_text=_("Your password, at least eight letters please"))

    def clean_email(self):
        email = self.cleaned_data["email"]
        if email:
            if User.objects.filter(username__iexact=email):
                raise forms.ValidationError(_("That email address is already used"))
        return email.lower()

    def clean_password(self):
        password = self.cleaned_data["password"]
        if password:
            if not len(password) >= 8:
                raise forms.ValidationError(_("Passwords must contain at least 8 letters."))
        return password


class OrgForm(forms.ModelForm):
    language = forms.ChoiceField(required=False, choices=[("", "")] + list(settings.LANGUAGES))
    timezone = TimeZoneFormField()

    def __init__(self, *args, **kwargs):
        super(OrgForm, self).__init__(*args, **kwargs)
        if "administrators" in self.fields:
            administrators = User.objects.exclude(username__in=["root", "root2"])
            administrators = administrators.exclude(pk__lt=0)
            self.fields["administrators"].queryset = administrators

    def clean_domain(self):
        domain = self.cleaned_data["domain"] or ""
        domain = domain.strip().lower()

        if domain:
            if domain == getattr(settings, "HOSTNAME", ""):
                raise forms.ValidationError(_("This domain is used for subdomains"))
            return domain
        else:
            return None

    class Meta:
        fields = forms.ALL_FIELDS
        model = Org



from smartmin.users.views import Login

from django.conf import settings
from django.contrib.auth.views import LogoutView
from django.urls import re_path

from . import views

logout_url = getattr(settings, "LOGOUT_REDIRECT_URL", None)

urlpatterns = [
    re_path(r"^login/$", Login.as_view(), dict(template_name="smartmin/users/login.html"), name="users.user_login"),
    re_path(
        r"^logout/$",
        LogoutView.as_view(),
        dict(redirect_field_name="go", next_page=logout_url),
        name="users.user_logout",
    ),
]

urlpatterns += views.UserCRUDL().as_urlpatterns()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-03-22 14:15


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0024_populate_org_backend")]

    operations = [
        migrations.AlterField(
            model_name="orgbackend",
            name="api_token",
            field=models.CharField(default="", help_text="The API token for this backend", max_length=128),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="orgbackend",
            name="backend_type",
            field=models.CharField(default="", max_length=256),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="orgbackend",
            name="host",
            field=models.CharField(default="", max_length=128),
            preserve_default=False,
        ),
        migrations.AlterUniqueTogether(name="orgbackend", unique_together=set([("org", "slug")])),
    ]



from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("categories", "0002_auto_20140820_1415"),
    ]

    operations = [
        migrations.CreateModel(
            name="CategoryImage",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(help_text="When this item was originally created", auto_now_add=True),
                ),
                ("modified_on", models.DateTimeField(help_text="When this item was last modified", auto_now=True)),
                ("name", models.CharField(help_text="The name to describe this image", max_length=64)),
                ("image", models.ImageField(help_text="The image file to use", upload_to="categories")),
                (
                    "category",
                    models.ForeignKey(
                        help_text="The category this image represents",
                        on_delete=models.PROTECT,
                        to="categories.Category",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"abstract": False},
            bases=(models.Model,),
        )
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("stories", "0010_auto_20140922_1514")]

    operations = [
        migrations.AddField(
            model_name="story",
            name="audio_link",
            field=models.URLField(
                help_text="A link to an mp3 file to publish on this story", max_length=255, null=True, blank=True
            ),
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-12 11:56


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("orgs", "0019_restructure_org_config")]

    operations = [migrations.RemoveField(model_name="org", name="api_token")]



from django.db import migrations


def populate_category_images(apps, schema_editor):
    Category = apps.get_model("categories", "Category")
    CategoryImage = apps.get_model("categories", "CategoryImage")
    User = apps.get_model("auth", "User")
    root = User.objects.filter(username="root").first()

    if not root:
        root = User.objects.filter(username="root2").first()

    if not root:
        root = User.objects.create(username="root2")

    for category in Category.objects.all():
        CategoryImage.objects.create(
            name=category.name, category=category, image=category.image, created_by=root, modified_by=root
        )


class Migration(migrations.Migration):

    dependencies = [("categories", "0003_categoryimage")]

    operations = [migrations.RunPython(populate_category_images)]



from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("categories", "0004_auto_20140904_0927")]

    operations = [
        migrations.AlterField(
            model_name="category",
            name="created_by",
            field=models.ForeignKey(
                related_name="categories_category_creations",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="category",
            name="modified_by",
            field=models.ForeignKey(
                related_name="categories_category_modifications",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which last modified this item",
            ),
        ),
        migrations.AlterField(
            model_name="categoryimage",
            name="category",
            field=models.ForeignKey(
                related_name="images",
                on_delete=models.PROTECT,
                to="categories.Category",
                help_text="The category this image represents",
            ),
        ),
        migrations.AlterField(
            model_name="categoryimage",
            name="created_by",
            field=models.ForeignKey(
                related_name="categories_categoryimage_creations",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which originally created this item",
            ),
        ),
        migrations.AlterField(
            model_name="categoryimage",
            name="modified_by",
            field=models.ForeignKey(
                related_name="categories_categoryimage_modifications",
                to=settings.AUTH_USER_MODEL,
                on_delete=models.PROTECT,
                help_text="The user which last modified this item",
            ),
        ),
    ]



# -*- coding: utf-8 -*-


from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("orgs", "0003_org_logo")]

    operations = [
        migrations.AlterField(
            model_name="org",
            name="administrators",
            field=models.ManyToManyField(
                help_text="The administrators in your organization",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Administrators",
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="editors",
            field=models.ManyToManyField(
                help_text="The editors in your organization", to=settings.AUTH_USER_MODEL, verbose_name="Editors"
            ),
        ),
        migrations.AlterField(
            model_name="org",
            name="viewers",
            field=models.ManyToManyField(
                help_text="The viewers in your organization", to=settings.AUTH_USER_MODEL, verbose_name="Viewers"
            ),
        ),
    ]



# -*- coding: utf-8 -*-


from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL), ("orgs", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="Story",
            fields=[
                ("id", models.AutoField(verbose_name="ID", serialize=False, auto_created=True, primary_key=True)),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="Whether this item is active, use this instead of deleting"
                    ),
                ),
                (
                    "created_on",
                    models.DateTimeField(help_text="When this item was originally created", auto_now_add=True),
                ),
                ("modified_on", models.DateTimeField(help_text="When this item was last modified", auto_now=True)),
                ("title", models.CharField(help_text="The title for this story", max_length=255)),
                ("featured", models.BooleanField(default=False, help_text="Whether this story is featured")),
                ("content", models.TextField(help_text="The body of text for the story")),
                (
                    "image",
                    models.ImageField(
                        help_text="Any image that should be displayed with this story", upload_to="stories"
                    ),
                ),
                (
                    "video_id",
                    models.CharField(
                        help_text="The id of the YouTube video that should be linked to this story",
                        max_length=255,
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "tags",
                    models.CharField(
                        help_text="Any tags for this story, separated by spaces, can be used to do more advanced filtering, optional",
                        max_length=255,
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        help_text="The user which originally created this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "modified_by",
                    models.ForeignKey(
                        help_text="The user which last modified this item",
                        on_delete=models.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "org",
                    models.ForeignKey(
                        help_text="The organization this story belongs to", on_delete=models.PROTECT, to="orgs.Org"
                    ),
                ),
            ],
            options={"verbose_name_plural": "Stories"},
            bases=(models.Model,),
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-15 08:23


import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("orgs", "0020_remove_org_api_token")]

    operations = [
        migrations.AlterField(
            model_name="org",
            name="config",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                default=dict,
                help_text="JSON blob used to store configuration information associated with this organization",
            ),
        )
    ]



from smartmin.users.views import UserCRUDL as SmartUserCRUDL


class UserCRUDL(SmartUserCRUDL):
    actions = ("create", "list", "update", "profile", "forget", "recover", "expired", "failed", "newpassword", "mimic")

    class Create(SmartUserCRUDL.Create):
        fields = ("username", "new_password", "first_name", "last_name", "email", "groups")

    class Update(SmartUserCRUDL.Update):
        fields = ("username", "new_password", "first_name", "last_name", "email", "is_active", "last_login", "groups")

    class Profile(SmartUserCRUDL.Profile):
        fields = (
            "username",
            "first_name",
            "last_name",
            "email",
            "old_password",
            "new_password",
            "confirm_new_password",
        )

        def pre_save(self, obj):
            obj = super(UserCRUDL.Profile, self).pre_save(obj)

            # keep our username and email in sync
            obj.username = obj.email

            return obj

        def has_permission(self, request, *args, **kwargs):
            return self.request.user.is_authenticated



from smartmin.views import SmartCreateView, SmartCRUDL, SmartDeleteView, SmartListView, SmartUpdateView

from django.db.models.functions import Lower
from django.utils.translation import gettext_lazy as _

from dash.orgs.views import OrgObjPermsMixin, OrgPermsMixin
from dash.tags.models import Tag


class TagCRUDL(SmartCRUDL):
    model = Tag
    actions = ("create", "list", "update", "delete")

    class Create(OrgPermsMixin, SmartCreateView):
        fields = ("name",)

        def pre_save(self, obj):
            obj = super(TagCRUDL.Create, self).pre_save(obj)

            org = self.derive_org()
            obj.org = org

            return obj

    class Update(OrgObjPermsMixin, SmartUpdateView):
        fields = ("name",)
        delete_url = "id@tags.tag_delete"

    class List(OrgPermsMixin, SmartListView):
        fields = ("name", "modified_on", "created_on")

        def get_queryset(self, **kwargs):
            queryset = super(TagCRUDL.List, self).get_queryset(**kwargs)
            queryset = queryset.filter(org=self.derive_org()).order_by(Lower("name"))

            return queryset

    class Delete(OrgObjPermsMixin, SmartDeleteView):
        cancel_url = "id@tags.tag_update"
        redirect_url = "@tags.tag_list"
        default_template = "smartmin/delete_confirm.html"
        submit_button_name = _("Delete")



from django.core.management.base import BaseCommand, CommandError
from django.utils import timezone

from dash.orgs.models import Org, TaskState


class Command(BaseCommand):
    help = "Manages org specific tasks"

    RUNNING = "running"
    FAILING = "failing"
    ENABLE = "enable"
    DISABLE = "disable"
    ACTION_CHOICES = (RUNNING, FAILING, ENABLE, DISABLE)

    def add_arguments(self, parser):
        parser.add_argument("action", choices=self.ACTION_CHOICES, help="The action to perform")
        parser.add_argument("task", nargs="?", default=None, help="Key of a task")
        parser.add_argument("org_ids", metavar="ORG", type=int, nargs="*", help="The ids of the orgs")

    def handle(self, *args, **options):
        org_ids = options["org_ids"]
        action = options["action"]
        task_key = options["task"]

        orgs = Org.objects.order_by("pk")
        if org_ids:
            orgs = orgs.filter(pk__in=org_ids)
        else:
            orgs = orgs.filter(is_active=True)

        if action in (self.ENABLE, self.DISABLE) and not task_key:
            raise CommandError("Must provide a task key")

        if action == self.RUNNING:
            self.do_running(orgs, task_key)
        elif action == self.FAILING:
            self.do_failing(orgs, task_key)
        elif action == self.ENABLE:
            self.do_enable(orgs, task_key)
        elif action == self.DISABLE:
            self.do_disable(orgs, task_key)

    def do_running(self, orgs, task_key):
        tasks = TaskState.objects.filter(org__in=orgs, ended_on=None).exclude(started_on=None)
        tasks = tasks.select_related("org").order_by("org_id", "task_key")
        if task_key:
            tasks = tasks.filter(task_key=task_key)

        self.render_task_table(tasks)

    def do_failing(self, orgs, task_key):
        tasks = TaskState.objects.filter(org__in=orgs, is_failing=True)
        tasks = tasks.select_related("org").order_by("org_id", "task_key")
        if task_key:
            tasks = tasks.filter(task_key=task_key)

        self.render_task_table(tasks)

    def do_enable(self, orgs, task_key):
        num_updated = 0
        for org in orgs:
            task = TaskState.objects.filter(org=org, task_key=task_key).first()
            if task:
                task.is_disabled = False
                task.save(update_fields=("is_disabled",))
                num_updated += 1
            else:
                self.stdout.write("No such task for org #%d" % org.pk)

        self.stdout.write("Task %s enabled for %d orgs" % (task_key, num_updated))

    def do_disable(self, orgs, task_key):
        num_updated = 0
        for org in orgs:
            task = TaskState.objects.filter(org=org, task_key=task_key).first()
            if task:
                task.is_disabled = True
                task.save(update_fields=("is_disabled",))
                num_updated += 1
            else:
                self.stdout.write("No such task for org #%d" % org.pk)

        self.stdout.write("Task %s disabled for %d orgs" % (task_key, num_updated))

    def render_task_table(self, tasks):
        header = [cell("Org ID", 8), cell("Org Name", 20), cell("Key", 20), cell("Started On", 20)]

        self.stdout.write("".join(header))
        self.stdout.write("==================================================================")

        for task in tasks:
            row = [
                cell(task.org.pk, 8),
                cell(task.org.name, 20),
                cell(task.task_key, 20),
                cell(format_date(task.started_on), 20),
            ]
            self.stdout.write("".join(row))


def cell(val, width):
    return str(val).ljust(width)


def format_date(dt):
    return dt.astimezone(timezone.utc).strftime("%b %d, %Y %H:%M") if dt else ""
